@@ -168,6 +168,23 @@ class Catalog(object):
                 break
             params['cursor'] = j['value']['cursor']
 
+    def get_rate_limit(self, cmd='derive.php'):
+        params = dict(rate_limits=1, cmd=cmd)
+        r = self.make_tasks_request(params)
+        line = ''
+        tasks = list()
+        for c in r.iter_content():
+            if six.PY3:
+                c = c.decode('utf-8')
+            if c == '\n':
+                j = json.loads(line)
+                task = CatalogTask(j, self)
+                tasks.append(task)
+                line = ''
+            line += c
+        j = json.loads(line)
+        return j
+
     def get_tasks(self, identifier=None, params=None):
         """Get a list of all tasks meeting all criteria.
         The list is ordered by submission time.
@@ -211,7 +228,11 @@ class Catalog(object):
         all_tasks = sorted(tasks, key=sort_by_date, reverse=True)
         return all_tasks
 
-    def submit_task(self, identifier, cmd, comment=None, priority=None, data=None):
+    def submit_task(self, identifier, cmd,
+                    comment=None,
+                    priority=None,
+                    data=None,
+                    headers=None):
         """Submit an archive.org task.
 
         :type identifier: str
@@ -235,6 +256,9 @@ class Catalog(object):
                      the request. Refer to `Tasks API Request Entity
                      <https://archive.org/services/docs/api/tasks.html#request-entity>`_.
 
+        :type headers: dict
+        :param headers: (optional) Add additional headers to request.
+
         :rtype: :class:`requests.Response`
         """
         data = dict() if not data else data
@@ -246,7 +270,11 @@ class Catalog(object):
                 data['args'] = dict(comment=comment)
         if priority:
             data['priority'] = priority
-        r = self.session.post(self.url, json=data, auth=self.auth, **self.request_kwargs)
+        r = self.session.post(self.url,
+                              json=data,
+                              auth=self.auth,
+                              headers=headers,
+                              **self.request_kwargs)
         return r
 
 
