@@ -24,34 +24,38 @@ usage:
     ia upload <identifier> - --remote-name=<name> [options]...
     ia upload <identifier> <file> --remote-name=<name> [options]...
     ia upload --spreadsheet=<metadata.csv> [options]...
+    ia upload <identifier> --file-metadata=<file_md.jsonl> [options]...
     ia upload <identifier> --status-check
     ia upload --help
 
 options:
     -h, --help
-    -q, --quiet                       Turn off ia's output [default: False].
-    -d, --debug                       Print S3 request parameters to stdout and exit
-                                      without sending request.
-    -r, --remote-name=<name>          When uploading data from stdin, this option sets the
-                                      remote filename.
-    -S, --spreadsheet=<metadata.csv>  bulk uploading.
-    -m, --metadata=<key:value>...     Metadata to add to your item.
-    -H, --header=<key:value>...       S3 HTTP headers to send with your request.
-    -c, --checksum                    Skip based on checksum. [default: False]
-    -v, --verify                      Verify that data was not corrupted traversing the
-                                      network. [default: False]
-    -n, --no-derive                   Do not derive uploaded files.
-    --size-hint=<size>                Specify a size-hint for your item.
-    --delete                          Delete files after verifying checksums
-                                      [default: False].
-    -R, --retries=<i>                 Number of times to retry request if S3 returns a
-                                      503 SlowDown error.
-    -s, --sleep=<i>                   The amount of time to sleep between retries
-                                      [default: 30].
-    --status-check                    Check if S3 is accepting requests to the given item.
-    --no-collection-check             Skip collection exists check [default: False].
-    -o, --open-after-upload           Open the details page for an item after upload
-                                      [default: False].
+    -q, --quiet                          Turn off ia's output [default: False].
+    -d, --debug                          Print S3 request parameters to stdout and exit
+                                         without sending request.
+    -r, --remote-name=<name>             When uploading data from stdin, this option sets
+                                         the remote filename.
+    -S, --spreadsheet=<metadata.csv>     Bulk uploading.
+    -f, --file-metadata=<file_md.jsonl>  Upload files with file-level metadata via a
+                                         file_md.jsonl file.
+    -m, --metadata=<key:value>...        Metadata to add to your item.
+    -H, --header=<key:value>...          S3 HTTP headers to send with your request.
+    -c, --checksum                       Skip based on checksum. [default: False]
+    -v, --verify                         Verify that data was not corrupted traversing the
+                                         network. [default: False]
+    -n, --no-derive                      Do not derive uploaded files.
+    --size-hint=<size>                   Specify a size-hint for your item.
+    --delete                             Delete files after verifying checksums
+                                         [default: False].
+    -R, --retries=<i>                    Number of times to retry request if S3 returns a
+                                         503 SlowDown error.
+    -s, --sleep=<i>                      The amount of time to sleep between retries
+                                         [default: 30].
+    --status-check                       Check if S3 is accepting requests to the given
+                                         item.
+    --no-collection-check                Skip collection exists check [default: False].
+    -o, --open-after-upload              Open the details page for an item after upload
+                                         [default: False].
 
 examples:
     ia upload <id> <file> -H x-archive-keep-old-version:0  # Turn off backups
@@ -64,6 +68,7 @@ import sys
 from tempfile import TemporaryFile
 from copy import deepcopy
 import webbrowser
+import json
 
 import six
 from docopt import docopt, printable_usage
@@ -72,7 +77,7 @@ from schema import Schema, Use, Or, And, SchemaError
 
 from internetarchive.cli.argparser import get_args_dict, convert_str_list_to_unicode
 from internetarchive.session import ArchiveSession
-from internetarchive.utils import validate_ia_identifier, get_s3_xml_text
+from internetarchive.utils import validate_s3_identifier, get_s3_xml_text, InvalidIdentifierException
 
 # Only import backports.csv for Python2 (in support of FreeBSD port).
 PY2 = sys.version_info[0] == 2
@@ -93,6 +98,9 @@ def _upload_files(item, files, upload_kwargs, prev_identifier=None, archive_sess
         responses += response
     except HTTPError as exc:
         responses += [exc.response]
+    except InvalidIdentifierException as exc:
+        print(str(exc))
+        sys.exit(1)
     finally:
         # Debug mode.
         if upload_kwargs['debug']:
@@ -130,7 +138,7 @@ def main(argv, session):
     # Validate args.
     s = Schema({
         str: Use(bool),
-        '<identifier>': Or(None, And(str, validate_ia_identifier,
+        '<identifier>': Or(None, And(str, validate_s3_identifier,
             error=('<identifier> should be between 3 and 80 characters in length, and '
                    'can only contain alphanumeric characters, periods ".", '
                    'underscores "_", or dashes "-". However, <identifier> cannot begin '
@@ -145,6 +153,8 @@ def main(argv, session):
             Use(lambda x: x.decode(sys.getfilesystemencoding()) if six.PY2 else x)),
         '--spreadsheet': Or(None, os.path.isfile,
                             error='--spreadsheet should be a readable file.'),
+        '--file-metadata': Or(None, os.path.isfile,
+                              error='--file-metadata should be a readable file.'),
         '--metadata': Or(None, And(Use(get_args_dict), dict),
                          error='--metadata must be formatted as --metadata="key:value"'),
         '--header': Or(None, And(Use(get_args_dict), dict),
@@ -168,10 +178,10 @@ def main(argv, session):
             collection_id = collection_id[0]
         collection = session.get_item(collection_id)
         if not collection.exists:
-            sys.stderr.write(
-                'You must upload to a collection that exists. '
-                '"{0}" does not exist.\n{1}\n'.format(collection_id,
-                                                      printable_usage(__doc__)))
+            print('You must upload to a collection that exists. '
+                  '"{0}" does not exist.\n{1}'.format(collection_id,
+                                                      printable_usage(__doc__)),
+                  file=sys.stderr)
             sys.exit(1)
 
     # Status check.
@@ -198,6 +208,14 @@ def main(argv, session):
     queue_derive = True if args['--no-derive'] is False else False
     verbose = True if args['--quiet'] is False else False
 
+    if args['--file-metadata']:
+        try:
+            args['<file>'] = json.load(open(args['--file-metadata']))
+        except json.decoder.JSONDecodeError:
+            args['<file>'] = list()
+            for line in open(args['--file-metadata']):
+                j = json.loads(line.strip())
+                args['<file>'].append(j)
     upload_kwargs = dict(
         metadata=args['--metadata'],
         headers=args['--header'],
@@ -209,6 +227,7 @@ def main(argv, session):
         retries=args['--retries'],
         retries_sleep=args['--sleep'],
         delete=args['--delete'],
+        validate_identifier=True
     )
 
     # Upload files.
@@ -245,7 +264,10 @@ def main(argv, session):
             prev_identifier = None
             for row in spreadsheet:
                 upload_kwargs_copy = deepcopy(upload_kwargs)
-                local_file = row['file']
+                if row.get('REMOTE_NAME'):
+                    local_file = {row['REMOTE_NAME']: row['file']}
+                else:
+                    local_file = row['file']
                 identifier = row.get('item', row.get('identifier'))
                 if not identifier:
                     print('error: no identifier column on spreadsheet!')
