@@ -2,7 +2,7 @@
 #
 # The internetarchive module is a Python/CLI interface to Archive.org.
 #
-# Copyright (C) 2012-2019 Internet Archive
+# Copyright (C) 2012-2021 Internet Archive
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -21,7 +21,7 @@
 internetarchive.iarequest
 ~~~~~~~~~~~~~~~~~~~~~~~~~
 
-:copyright: (C) 2012-2019 by Internet Archive.
+:copyright: (C) 2012-2021 by Internet Archive.
 :license: AGPL 3, see LICENSE for more details.
 """
 from __future__ import absolute_import
@@ -32,6 +32,7 @@ except ImportError:
     import json
 import re
 import copy
+import logging
 
 from six.moves import urllib
 import requests.models
@@ -40,12 +41,17 @@ from jsonpatch import make_patch
 import six
 
 from internetarchive import auth, __version__
-from internetarchive.utils import needs_quote
+from internetarchive.utils import needs_quote, delete_items_from_dict
+from internetarchive.exceptions import ItemLocateError
+
+
+logger = logging.getLogger(__name__)
 
 
 class S3Request(requests.models.Request):
     def __init__(self,
                  metadata=None,
+                 file_metadata=None,
                  queue_derive=True,
                  access_key=None,
                  secret_key=None,
@@ -60,6 +66,7 @@ class S3Request(requests.models.Request):
         metadata = {} if metadata is None else metadata
 
         self.metadata = metadata
+        self.file_metadata = file_metadata
         self.queue_derive = queue_derive
 
     def prepare(self):
@@ -77,6 +84,7 @@ class S3Request(requests.models.Request):
 
             # S3Request kwargs.
             metadata=self.metadata,
+            file_metadata=self.file_metadata,
             queue_derive=self.queue_derive,
         )
         return p
@@ -90,10 +98,12 @@ class S3PreparedRequest(requests.models.PreparedRequest):
 
     def prepare(self, method=None, url=None, headers=None, files=None, data=None,
                 params=None, auth=None, cookies=None, hooks=None, queue_derive=None,
-                metadata={}):
+                metadata=None, file_metadata=None):
         self.prepare_method(method)
         self.prepare_url(url, params)
-        self.prepare_headers(headers, metadata, queue_derive)
+        self.prepare_headers(headers, metadata,
+                             file_metadata=file_metadata,
+                             queue_derive=queue_derive)
         self.prepare_cookies(cookies)
         self.prepare_body(data, files)
         self.prepare_auth(auth, url)
@@ -103,7 +113,7 @@ class S3PreparedRequest(requests.models.PreparedRequest):
         # This MUST go after prepare_auth. Authenticators could add a hook
         self.prepare_hooks(hooks)
 
-    def prepare_headers(self, headers, metadata, queue_derive=True):
+    def prepare_headers(self, headers, metadata, file_metadata=None, queue_derive=True):
         """Convert a dictionary of metadata into S3 compatible HTTP
         headers, and append headers to ``headers``.
 
@@ -115,42 +125,54 @@ class S3PreparedRequest(requests.models.PreparedRequest):
         :param headers: (optional) S3 compatible HTTP headers.
 
         """
+        metadata = dict() if metadata is None else metadata
+        file_metadata = dict() if file_metadata is None else file_metadata
+
         if not metadata.get('scanner'):
             scanner = 'Internet Archive Python library {0}'.format(__version__)
             metadata['scanner'] = scanner
         prepared_metadata = prepare_metadata(metadata)
+        prepared_file_metadata = prepare_metadata(file_metadata)
 
         headers['x-archive-auto-make-bucket'] = '1'
-        if queue_derive is False:
-            headers['x-archive-queue-derive'] = '0'
-        else:
-            headers['x-archive-queue-derive'] = '1'
+        if 'x-archive-queue-derive' not in headers:
+            if queue_derive is False:
+                headers['x-archive-queue-derive'] = '0'
+            else:
+                headers['x-archive-queue-derive'] = '1'
+
+        def _prepare_metadata_headers(prepared_metadata, meta_type='meta'):
+            for meta_key, meta_value in prepared_metadata.items():
+                # Encode arrays into JSON strings because Archive.org does not
+                # yet support complex metadata structures in
+                # <identifier>_meta.xml.
+                if isinstance(meta_value, dict):
+                    meta_value = json.dumps(meta_value)
+                # Convert the metadata value into a list if it is not already
+                # iterable.
+                if (isinstance(meta_value, six.string_types)
+                        or not hasattr(meta_value, '__iter__')):
+                    meta_value = [meta_value]
+                # Convert metadata items into HTTP headers and add to
+                # ``headers`` dict.
+                for i, value in enumerate(meta_value):
+                    if not value:
+                        continue
+                    header_key = 'x-archive-{0}{1:02d}-{2}'.format(meta_type, i, meta_key)
+                    if (isinstance(value, six.string_types) and needs_quote(value)):
+                        if six.PY2 and isinstance(value, six.text_type):
+                            value = value.encode('utf-8')
+                        value = 'uri({0})'.format(urllib.parse.quote(value))
+                    # because rfc822 http headers disallow _ in names, IA-S3 will
+                    # translate two hyphens in a row (--) into an underscore (_).
+                    header_key = header_key.replace('_', '--')
+                    headers[header_key] = value
+
+        # Parse the prepared metadata into HTTP headers,
+        # and add them to the ``headers`` dict.
+        _prepare_metadata_headers(prepared_metadata)
+        _prepare_metadata_headers(prepared_file_metadata, meta_type='filemeta')
 
-        for meta_key, meta_value in prepared_metadata.items():
-            # Encode arrays into JSON strings because Archive.org does not
-            # yet support complex metadata structures in
-            # <identifier>_meta.xml.
-            if isinstance(meta_value, dict):
-                meta_value = json.dumps(meta_value)
-            # Convert the metadata value into a list if it is not already
-            # iterable.
-            if (isinstance(meta_value, six.string_types) or
-                    not hasattr(meta_value, '__iter__')):
-                meta_value = [meta_value]
-            # Convert metadata items into HTTP headers and add to
-            # ``headers`` dict.
-            for i, value in enumerate(meta_value):
-                if not value:
-                    continue
-                header_key = 'x-archive-meta{0:02d}-{1}'.format(i, meta_key)
-                if (isinstance(value, six.string_types) and needs_quote(value)):
-                    if six.PY2 and isinstance(value, six.text_type):
-                        value = value.encode('utf-8')
-                    value = 'uri({0})'.format(urllib.parse.quote(value))
-                # because rfc822 http headers disallow _ in names, IA-S3 will
-                # translate two hyphens in a row (--) into an underscore (_).
-                header_key = header_key.replace('_', '--')
-                headers[header_key] = value
         super(S3PreparedRequest, self).prepare_headers(headers)
 
 
@@ -232,80 +254,125 @@ class MetadataPreparedRequest(requests.models.PreparedRequest):
 
         if not source_metadata:
             r = requests.get(self.url)
-            source_metadata = r.json().get(target.split('/')[0], {})
+            source_metadata = r.json()
 
         # Write to many targets
-        if any('/' in k for k in metadata):
-            if not all('/' in k for k in metadata if k != 'metadata'):
-                raise ValueError('bad!')
+        if (isinstance(metadata, list)
+                or any('/' in k for k in metadata)
+                or all(isinstance(k, dict) for k in metadata.values())):
             changes = list()
+
+            if any(not k for k in metadata):
+                raise ValueError('Invalid metadata provided, '
+                                 'check your input and try again')
+
+            if target:
+                metadata = {target: metadata}
             for key in metadata:
                 if key == 'metadata':
-                    patch = prepare_metadata_patch(metadata[key],
-                                                   source_metadata['metadata'],
-                                                   append,
-                                                   append_list)
-                elif key.startswith('files/'):
+                    try:
+                        patch = prepare_patch(metadata[key],
+                                              source_metadata['metadata'],
+                                              append,
+                                              append_list)
+                    except KeyError:
+                        raise ItemLocateError
+                elif key.startswith('files'):
                     patch = prepare_files_patch(metadata[key],
                                                 source_metadata['files'],
                                                 append,
                                                 key,
                                                 append_list)
-                if patch:
-                    changes.append({'target': key, 'patch': patch})
+                else:
+                    key = key.split('/')[0]
+                    patch = prepare_target_patch(metadata, source_metadata, append,
+                                                 target, append_list, key)
+                changes.append({'target': key, 'patch': patch})
             self.data = {
                 '-changes': json.dumps(changes),
                 'priority': priority,
             }
+            logger.debug('submitting metadata request: {}'.format(self.data))
         # Write to single target
         else:
-            if 'metadata' in target:
-                patch = prepare_metadata_patch(metadata,
-                                               source_metadata,
-                                               append,
-                                               append_list)
+            if not target or 'metadata' in target:
+                target = 'metadata'
+                try:
+                    patch = prepare_patch(metadata, source_metadata['metadata'], append,
+                                          append_list)
+                except KeyError:
+                    raise ItemLocateError
             elif 'files' in target:
-                patch = prepare_files_patch(metadata, source_metadata, append,
+                patch = prepare_files_patch(metadata, source_metadata['files'], append,
                                             target, append_list)
             else:
-                destination_metadata = source_metadata.copy()
-                prepared_metadata = prepare_metadata(metadata, source_metadata, append)
-                destination_metadata.update(prepared_metadata)
+                metadata = {target: metadata}
+                patch = prepare_target_patch(metadata, source_metadata, append,
+                                             target, append_list, target)
             self.data = {
                 '-patch': json.dumps(patch),
                 '-target': target,
                 'priority': priority,
             }
+            logger.debug('submitting metadata request: {}'.format(self.data))
         super(MetadataPreparedRequest, self).prepare_body(self.data, None)
 
 
+def prepare_patch(metadata, source_metadata, append, append_list=None):
+    destination_metadata = source_metadata.copy()
+    if isinstance(metadata, list):
+        prepared_metadata = metadata
+        if not destination_metadata:
+            destination_metadata = list()
+    else:
+        prepared_metadata = prepare_metadata(metadata, source_metadata, append,
+                                             append_list)
+    if isinstance(destination_metadata, dict):
+        destination_metadata.update(prepared_metadata)
+    elif isinstance(metadata, list) and not destination_metadata:
+        destination_metadata = metadata
+    else:
+        if isinstance(prepared_metadata, list):
+            if append_list:
+                destination_metadata += prepared_metadata
+            else:
+                destination_metadata = prepared_metadata
+        else:
+            destination_metadata.append(prepared_metadata)
+    # Delete metadata items where value is REMOVE_TAG.
+    destination_metadata = delete_items_from_dict(destination_metadata, 'REMOVE_TAG')
+    patch = make_patch(source_metadata, destination_metadata).patch
+    return patch
+
+
+def prepare_target_patch(metadata, source_metadata, append, target, append_list, key):
+
+    def dictify(lst, key=None, value=None):
+        if not lst:
+            return value
+        sub_dict = dictify(lst[1:], key, value)
+        for i, v in enumerate(lst):
+            md = {v: copy.deepcopy(sub_dict)}
+            return md
+
+    for _k in metadata:
+        metadata = dictify(_k.split('/')[1:], _k.split('/')[-1], metadata[_k])
+    for i, _k in enumerate(key.split('/')):
+        if i == 0:
+            source_metadata = source_metadata.get(_k, dict())
+        else:
+            source_metadata[_k] = source_metadata.get(_k, dict()).get(_k, dict())
+    patch = prepare_patch(metadata, source_metadata, append, append_list)
+    return patch
+
+
 def prepare_files_patch(metadata, source_metadata, append, target, append_list):
     filename = '/'.join(target.split('/')[1:])
     for f in source_metadata:
         if f.get('name') == filename:
             source_metadata = f
             break
-    destination_metadata = source_metadata.copy()
-    prepared_metadata = prepare_metadata(metadata, source_metadata, append, append_list)
-    destination_metadata.update(prepared_metadata)
-    # Delete metadata items where value is REMOVE_TAG.
-    destination_metadata = dict(
-        (k, v) for (k, v) in destination_metadata.items() if v != 'REMOVE_TAG'
-    )
-    patch = make_patch(source_metadata, destination_metadata).patch
-    return patch
-
-
-def prepare_metadata_patch(metadata, source_metadata, append, append_list):
-    destination_metadata = source_metadata.copy()
-    prepared_metadata = prepare_metadata(metadata, source_metadata, append,
-                                         append_list)
-    destination_metadata.update(prepared_metadata)
-    # Delete metadata items where value is REMOVE_TAG.
-    destination_metadata = dict(
-        (k, v) for (k, v) in destination_metadata.items() if v != 'REMOVE_TAG'
-    )
-    patch = make_patch(source_metadata, destination_metadata).patch
+    patch = prepare_patch(metadata, source_metadata, append, append_list)
     return patch
 
 
@@ -379,8 +446,12 @@ def prepare_metadata(metadata, source_metadata=None, append=False, append_list=F
             if not isinstance(metadata[key], list):
                 metadata[key] = [metadata[key]]
             for v in metadata[key]:
-                if v in source_metadata[key]:
-                    continue
+                if not isinstance(source_metadata[key], list):
+                    if v in [source_metadata[key]]:
+                        continue
+                else:
+                    if v in source_metadata[key]:
+                        continue
                 if not isinstance(source_metadata[key], list):
                     prepared_metadata[key] = [source_metadata[key]]
                 else:
