@@ -287,6 +287,8 @@ class ArchiveSession(requests.sessions.Session):
                      fields=None,
                      sorts=None,
                      params=None,
+                     full_text_search=None,
+                     dsl_fts=None,
                      request_kwargs=None,
                      max_retries=None):
         """Search for items on Archive.org.
@@ -303,6 +305,15 @@ class ArchiveSession(requests.sessions.Session):
         :param params: (optional) The URL parameters to send with each request sent to the
                        Archive.org Advancedsearch Api.
 
+        :type full_text_search: bool
+        :param full_text_search: (optional) Beta support for querying the archive.org
+                                 Full Text Search API [default: False].
+
+        :type dsl_fts: bool
+        :param dsl_fts: (optional) Beta support for querying the archive.org Full Text
+                        Search API in dsl (i.e. do not prepend ``!L `` to the
+                        ``full_text_search`` query [default: False].
+
         :returns: A :class:`Search` object, yielding search results.
         """
         request_kwargs = {} if not request_kwargs else request_kwargs
@@ -310,6 +321,8 @@ class ArchiveSession(requests.sessions.Session):
                       fields=fields,
                       sorts=sorts,
                       params=params,
+                      full_text_search=full_text_search,
+                      dsl_fts=dsl_fts,
                       request_kwargs=request_kwargs,
                       max_retries=max_retries)
 
@@ -337,8 +350,13 @@ class ArchiveSession(requests.sessions.Session):
         else:
             return True
 
+    def get_tasks_api_rate_limit(self, cmd='derive.php', request_kwargs=None):
+        c = Catalog(self, request_kwargs)
+        r = c.get_rate_limit(cmd=cmd)
+        return r
+
     def submit_task(self, identifier, cmd, comment=None, priority=None, data=None,
-                    request_kwargs=None):
+                    headers=None, reduced_priority=None, request_kwargs=None):
         """Submit an archive.org task.
 
         :type identifier: str
@@ -362,14 +380,34 @@ class ArchiveSession(requests.sessions.Session):
                      the request. Refer to `Tasks API Request Entity
                      <https://archive.org/services/docs/api/tasks.html#request-entity>`_.
 
+        :type headers: dict
+        :param headers: (optional) Add additional headers to request.
+
+        :type reduced_priority: bool
+        :param reduced_priority: (optional) Submit your derive at a lower priority.
+                                 This option is helpful to get around rate-limiting.
+                                 Your task will more likey be accepted, but it might
+                                 not run for a long time. Note that you still may be
+                                 subject to rate-limiting. This is different than
+                                 ``priority`` in that it will allow you to possibly
+                                 avoid rate-limiting.
+
         :type request_kwargs: dict
         :param request_kwargs: (optional) Keyword arguments to be used in
                                :meth:`requests.sessions.Session.post` request.
 
         :rtype: :class:`requests.Response`
         """
+        headers = dict() if not headers else headers
+        if reduced_priority is not None:
+            headers.update({'X-Accept-Reduced-Priority': '1'})
+
         c = Catalog(self, request_kwargs)
-        r = c.submit_task(identifier, cmd, comment=comment, priority=priority, data=data)
+        r = c.submit_task(identifier, cmd,
+                          comment=comment,
+                          priority=priority,
+                          data=data,
+                          headers=headers)
         return r
 
     def iter_history(self, identifier, params=None, request_kwargs=None):
